package mi.hdm;

import javafx.application.Application;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.stage.Stage;
import mi.hdm.controllers.View;
import mi.hdm.filesystem.FileManager;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.InputStream;

public class GUIDriver extends Application {
    private final static Logger log = LogManager.getLogger(GUIDriver.class);

    private final static View STARTUP_VIEW = View.MAIN;
    private final static String PATH_TO_ICON = "/images/Tasty_Pages_Icon.png";

    private static Stage stage;
    private static TastyPages model;

    public static void main(String[] args) {
        log.info("Starting TastyPages");
        try {
            model = FileManager.deserializeFromFilesystem();
            //model.createRecipe();
        } catch (Exception e) {
            log.fatal("Internal exception, check file paths and URIs");
            e.printStackTrace();
            System.exit(1);
        }
        launch(args);
        log.info("Shutting down application");
        log.info("Saving all data");
        FileManager.serializeToFile(model);
    }

    @Override
    public void start(Stage stage) throws Exception {
        GUIDriver.stage = stage;

        Parent parent = STARTUP_VIEW.getScene();
        final Scene scene = new Scene(parent, 1400, 800);

        stage.setTitle(STARTUP_VIEW.getWindowTitle());
        stage.setMaximized(true); //open scene in full screen
        stage.setScene(scene);
        stage.show();

        InputStream iconStream = getClass().getResourceAsStream(PATH_TO_ICON);
        if (iconStream != null) {
            stage.getIcons().add(new Image(iconStream));
            log.debug("Set icon for application");
        } else log.warn("Could not find icon for application. Check the filepath");
    }

    public static TastyPages getModel() {
        return model;
    }

    public static Stage getApplicationStage() {
        return stage;
    }

    public static View getStartupView() {
        return STARTUP_VIEW;
    }
}
